#ifndef _BLASR_STRING_TO_SCORE_MATRIX_HPP_
#define _BLASR_STRING_TO_SCORE_MATRIX_HPP_

#include <string>
#include <sstream>
#include <vector>

bool StringToScoreMatrix(std::string &str, int matrix[5][5]);
#endif
