#ifndef _BLASR_SCORE_MATRICES_HPP_
#define _BLASR_SCORE_MATRICES_HPP_

extern int QVDistanceMatrix[5][5]; 

extern int EditDistanceMatrix[5][5]; 

extern int SMRTDistanceMatrix[5][5]; 

extern int SMRTLogProbMatrix[5][5]; 

extern int LowMutationMatrix[5][5]; 

extern int LocalAlignLowMutationMatrix[5][5]; 

extern int CrossMatchMatrix[5][5];

#endif // _BLASR_SCORE_MATRICES_HPP_
