#ifndef ALIGNED_PAIR_H_
#define ALIGNED_PAIR_H_

#include <string>

template<typename T_AlignChar=char>
class AlignedPair {
 public:
	int length;
	T_AlignChar *tAlignmentSeq;
	T_AlignChar *qAlignmentSeq;
};


inline
void ReadCompareSequencesAlignment {

}
#endif
