/* * ============================================================================
 *
 *       Filename:  testdata.h
 *
 *    Description:  List files used in unit tests.
 *
 *        Version:  1.0
 *        Created:  08/22/2013 11:10:54 AM
 *       Revision:  none
 *       Compiler:  gcc
 *
 *         Author:  Yuan Li (yli), yli@pacificbiosciences.com
 *        Company:  Pacific Biosciences
 *
 * ============================================================================
 */
#include <string>

using namespace std;

const string baxFile1 = 
    ("/mnt/secondary-siv/testdata/BlasrTestData/ctest/data/"
     "testLoadPulses/Analysis_Results/"
     "m130302_011223_pd1_c000000092559900001500000112311511_s1_p0.1.bax.h5");

const string baxFile2 = 
    ("/mnt/secondary-siv/testdata/BlasrTestData/ctest/data/"
     "testLoadPulses/Analysis_Results/"
     "m130302_011223_pd1_c000000092559900001500000112311511_s1_p0.2.bax.h5");

const string baxFile3 = 
    ("/mnt/secondary-siv/testdata/BlasrTestData/utest/data/"
     "test_read_chemistry_triple/"
     "m150223_190837_42175_c100735112550000001823160806051530_s1_p0.1.bax.h5");

const string plsFile1 = 
    ("/mnt/secondary-siv/testdata/BlasrTestData/ctest/data/"
     "testLoadPulses/Analysis_Results/"
     "m121215_065521_richard_c100425710150000001823055001121371_s1_p0.pls.h5");

const string ccsFile1 = 
    ("/mnt/secondary-siv/testdata/BlasrTestData/utest/data/ccs_P4-C2/"
     "m130328_211423_ethan_c100499512550000001823070408081371_s1_p0.ccs.h5");

const string fastaFile1 = 
    ("/mnt/secondary-siv/testdata/BlasrTestData/utest/"
     "data/read.fasta");

const string fastqFile1 = 
    ("/mnt/secondary-siv/testdata/BlasrTestData/utest/data/ccs_P4-C2/"
     "m130328_211423_ethan_c100499512550000001823070408081371_s1_p0.fastq");

const string titleTable1 = 
    ("/mnt/secondary-siv/testdata/BlasrTestData/utest/"
     "data/titleTable1");
 
const string rgnFile1 =
    ("/mnt/secondary-siv/testdata/BlasrTestData/utest/"
     "data/test_regionTable.rgn.h5");

const string scanDataFile1 = 
    ("/mnt/secondary-siv/testdata/BlasrTestData/utest/"
     "data/scandata.h5");

const string bamFile1 = 
    ("/mnt/secondary-siv/testdata/BlasrTestData/ctest/data/test_bam/"
     "m140905_042212_sidney_c100564852550000001823085912221377_s1_X0.subreads.bam");

const string bamFile2 =   
    ("/mnt/secondary-siv/testdata/BlasrTestData/ctest/data/test_bam/"
     "m150325_224749_42269_c100795290850000001823159309091522_s1_p0.subreads.bam");
